from typing import Any
from selenium.webdriver.remote.webelement import WebElement

from ..MicrosoftFormComponent import MicrosoftFormComponent


class Radio(MicrosoftFormComponent):
    """Create a Microsoft form Radio (select with no dropdown) component

    Paramaters
    ----------
    web_element: `'WebElement'`, default `None`
        web element associated with form component    
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        super().__init__(web_element=web_element)

    def fill_in(self, response: str):
        """Select the right answer

        Paramaters
        ----------
        response: `str`
            the right answer
            (if options are: `["Option1", "Option 2"] you will` pass `"Option 1"` if it is the right one)
        """
        answers_container = self.web_element.find_element_by_class_name(
            'office-form-question-element')
        answer = answers_container.find_element_by_xpath(
            ".//*[contains(text(), '{}')]".format(response))
        answer.click()



from datetime import datetime
from typing import List, Optional
from O365.calendar import Event as MicrosoftEvent
import random
import string

from .Task import Task


class Event:
    """Create a new Event to add to calendar

    Paramaters
    ----------
    title: str
        title of the event
    start_date: `'datetime'`
        when the event starts
    end_date: `'datetime'`
        when the event ends
    tasks: `List['Task']`
        list of tasks to execute for this event
    id: `Optional[str]`, default `None`
        id of the event (if not provided a random 16 chars string)
    """

    def __init__(self, start_date: 'datetime', end_date: 'datetime',  title: str, tasks: List['Task'] = [], id: Optional[str] = None) -> None:
        if id != None:
            self.id = id
        else:
            self.id = self.__generate_id()

        self.start_date = start_date
        self.end_date = end_date
        self.title = title
        self.tasks = tasks

    def add_tasks(self, tasks: List['Task']) -> None:
        """Add a list of tasks to event

        Paramaters
        ----------

        tasks: `List['Task']`
            tasks to add to end of event.tasks list

        Returns
        -------
        `None`
        """
        self.tasks.extend(tasks)

    def add_task(self, task: 'Task') -> None:
        """Add taks to event

        Paramaters
        ----------
        task: `'Task'`
            task to ass to end of event.tasks list

        Returns
        -------
        `None`
        """
        self.tasks.append(task)

    def from_microsoft_event(micro_event: 'MicrosoftEvent') -> 'Event':
        """Creates `Event` object from `MicrosoftEvent` (returned from API when using `MicrosoftSource`)

        Paramaters
        ----------
        micro_event: `'MicrosoftEvent'`
            obj returned from API (when using `MicrosoftSource`)

        Returns
        -------
        `'Event'`
        """
        subject = micro_event.subject
        start = micro_event.start
        end = micro_event.end
        tasks = []
        return Event(start_date=start, end_date=end, title=subject, tasks=tasks)

    def __generate_id(self, length: int = 16) -> str:
        """Generates randome string of length

        Paramaters
        ----------
        length: `int`, default `16`
            length of retuned string

        Returns
        -------
        `str`
            random generated id
        """

        return ''.join(random.SystemRandom().choice(string.ascii_lowercase + string.digits) for _ in range(length))

    def __eq__(self, other: 'Event') -> bool:
        """Verify if object are equal

        Paramaters
        ----------
        other: `'Event'`
            event to compare `self` with

        Returns
        -------
        bool
        """
        return self.id == other.id

    def __str__(self) -> str:
        return "(Event){ " + "Title: " + self.title.upper() + ", Trigger date: " + "#Tasks: " + str(len(self.tasks)) + " }"



import os
from time import sleep
from O365.account import Account
from O365.utils.token import FileSystemTokenBackend
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from typing import Callable, List, Optional, Tuple, TypedDict
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException
import json
from datetime import datetime


from .Event import Event
from .Source import Source
from .deafults import default_token_file, default_store_path


class AccountCred(TypedDict):
    email: str
    password: str


class MicrosoftSource(Source):
    """Create a source subclass that uses Microsoft graph API to fetch events

    Paramaters
    ----------
    path_to_driver: `str`
        required path to chrome driver (default one)
    api_credentials: `Tuple[str, str]`
        (client_id, api_secret) Microsoft API credentials
    account_cred: `'AccountCred'`
        (email, password) your Microsoft account credentials
    driver: `'WebDriver'`, default `None`
        optional driver used instead of the deafult one
    path_to_store: `str`, default: `""`
        path to file storage (folder storing tokens, driver data...).
        If not provided the store path will be `../` from this directory
    scopes: `List[str]`, default `[]`
        scopes used to call Microsoft API    
    """

    def __init__(self, path_to_driver: str, api_credentials: Tuple[str, str], account_cred: 'AccountCred', driver: 'WebDriver' = None, path_to_store: str = "", scopes: List[str] = []) -> None:
        super(MicrosoftSource, self).__init__(driver=driver,
                                              path_to_store=path_to_store, path_to_driver=path_to_driver)

        self.scopes = scopes
        self.api_credentials = api_credentials
        self.account_cred = account_cred

    def load_events(self, filter: Optional[Callable[['Event'], bool]] = None, map: Optional[Callable[['MicrosoftSource', 'Event'], 'Event']] = None) -> List['Event']:
        """Load events from API

        Paramaters
        ----------
        filter: `Optional[Callable[['Event'], bool]]`, default `None`
            callback function to filter not wanted events
        map: `Optional[Callable[['MicrosoftSource', 'Event'], 'Event']]`, default `None`
            callback function to modify previously filtered events (for instance to add a task based on the event)

        Returns
        -------
        `List['Event']`
            events fetched, filtered and mapped
        """
        token_backend = FileSystemTokenBackend(
            token_path=self.path_to_store, token_filename=default_token_file)
        account = Account(credentials=self.api_credentials,
                          token_backend=token_backend)
        should_login = not self.__is_in_session()

        if should_login:
            if not self.__api_authenticate(account=account):
                return

        if should_login:
            print('****** Authenticated ******')

        schedule = account.schedule()
        calendar = schedule.get_default_calendar()
        events = calendar.get_events(include_recurring=False)

        parsed_events: List['Event'] = []
        for i, mic_event in enumerate(events):
            event = Event.from_microsoft_event(mic_event)
            changed_event = event
            to_add = True
            if filter != None:
                if not filter(event):
                    to_add = False

            if map != None and to_add:
                changed_event = map(source=self, event=event)

            if to_add:
                parsed_events.insert(i + 1, changed_event)

        return parsed_events

    def __is_in_session(self) -> bool:
        """Find out if the current session is expired

        Returns
        -------
        `bool`
            `True` if valid session (logged in) 
        """

        in_session = False
        token_path = os.path.join(self.path_to_store, default_token_file)
        if os.path.exists(token_path):
            with open(token_path, "r") as f:
                token = json.load(f)
                token_expiration = token['expires_at']
                now = datetime.now().timestamp()
                in_session = token_expiration > now
                f.close()

        return in_session

    def __api_authenticate(self, account: 'Account') -> bool:
        """Uses O365 library to authenticate using scopes and handle_consent

        Returns
        -------

        `bool`
            `True` if authentication is successful
        """

        return account.authenticate(scopes=self.scopes, handle_consent=self.login)

    # change name to login (also in Source superclass)
    def login(self, consent_url: Optional[str] = None) -> str:
        """Logins to Microsoft account. 
        Called by `account.authenticate(scopes=scopes, handle_consent=self.handle_consent)` (O365 package) to login with command line.
        If consent_url is not passed it will login normally (calls `__selenium_login`)


        Paramaters
        ----------
        consent_url: `Optional[str]`, default `None`
            url used to command line login

        Returns
        -------
        `str`
            url after login
        """

        if consent_url != None:
            self.driver.get(consent_url)
        sleep(2)
        # find out if if stat is needed should't be cause if method runs should need authentication
        if self.needs_login(self.driver.current_url):
            return self.__selenium_login()
        else:
            return self.driver.current_url

    def __selenium_login(self) -> str:
        """Logins to Microsoft account using selenium automation

        Returns
        -------
        `str`
            url after login
        """

        print("Logging in using credentials")
        account_selector = None
        try:
            account_selector = self.driver.find_element_by_id('tilesHolder')
        except NoSuchElementException:
            pass

        if account_selector != None:
            self.__select_account(account_selector=account_selector)
        else:

            sleep(2)
            email_input = self.driver.find_element_by_id("i0116")
            self.__complete_input_field(
                input=email_input, input_text=self.account_cred['email'])

        sleep(2)

        password_input = self.driver.find_element_by_id("i0118")
        self.__complete_input_field(
            input=password_input, input_text=self.account_cred['password'])

        sleep(2)

        if self.__is2FA():
            self.__handle_2FA()

        sleep(2)

        if(self.__is_asking_app_permission(current_url=self.driver.current_url)):
            self.__handle_app_permission()

        sleep(2)

        return self.driver.current_url

    def __is_asking_app_permission(self, current_url: str) -> bool:
        """Returns boolean based on if microsoft is asking to give app (Azure app) permission to access profile.
        Usually asks only if is first time running the client (on specific account)

        Returns
        -------
        `bool`
            `True` if is asking to give permission to app
        """

        return "/common/login" in current_url

    def __handle_app_permission(self) -> None:
        """Gives permission to app

        Returns
        -------
        `None`
        """

        confirm_btn = self.driver.find_element_by_id('idSIButton9')
        confirm_btn.click()

    def __complete_input_field(self, input: 'WebElement', input_text: str) -> None:
        """Complete text field using selenium

        Paramaters
        ----------
        input: `'WebElement'`
            selenium element
        input_text: `str`
            string to be inputted in text field

        Returns
        -------
        `None`
        """

        input.clear()
        input.send_keys(input_text)
        input.send_keys(Keys.RETURN)

    def __select_account(self, account_selector: 'WebElement') -> None:
        """Selects account in account selector page (using selenium)

        Paramaters
        ----------
        account_selector: `'WebElement'`

        Returns
        -------
        `None`
        """

        account_selector.find_elements_by_xpath(
            ".//*[contains(text(), '{}')]".format(self.account_cred['email']))
        account_selector.click()

    def __is2FA(self) -> bool:
        """Retuned boolean based on if 2FA confirmation is required (after email and password inputted)

        Returns
        -------
        `bool`
            `True` if 2FA confirmation is required
        """

        return "DeviceAuthTls" in self.driver.current_url

    def __handle_2FA(self) -> None:
        """Handles 2FA using selenium. Asks to input the OTP in the terminal

        Returns 
        -------
        `None`
        """

        msg_box = self.driver.find_element_by_id(
            'idDiv_SAOTCC_Description')

        msg = msg_box.text
        code = input(msg + "\n")
        code_input = self.driver.find_element_by_id('idTxtBx_SAOTCC_OTC')

        code_input.clear()
        code_input.send_keys(code)

        disable_2fa_check_box = self.driver.find_element_by_id(
            'idChkBx_SAOTCC_TD')
        disable_2fa_check_box.click()

        code_input.send_keys(Keys.RETURN)

    # try using it in Form
    def needs_login(self, current_url: Optional[str] = None) -> bool:
        """Retuns boolen based on if login is required

        Paramaters
        ----------
        current_url: `Optional[str]`, default `None`
            driver url (if not passed uses self.driver.current_url)
        Returns
        -------
        `bool`
            `True` if login is required
        """

        return "login" in current_url if current_url != None else self.driver.current_url



from datetime import datetime
from typing import Any, Callable, Optional
import string
import random

from .deafults import default_callback


class Task:
    """Create task used in `Event`

    Paramaters
    ----------
    id: `Optional[str]`, default `None`
        id of tasks (if not provided 16 chars random string)
    run_date: `'datetime'` 
        datetime when task will be run
    to_run: `Callable[[], bool]`, default `default_callback`
        callback function that will run when run_date is now
    on_success: Optional[Callable[[Any], None]], default `default_callback`
        callback function runned if `to_run` return `True`
    on_failure: Optional[Callable[[Any], None]], default `default_callback`
        callback function runned if `to_run` returns `False`
    """

    def __init__(self, run_date: 'datetime', id: Optional[str] = None, to_run: Callable[[], bool] = default_callback, on_success: Optional[Callable[[Any], None]] = default_callback, on_failure: Optional[Callable[[Any], None]] = default_callback) -> None:
        if id != None:
            self.id = id
        else:
            self.id = self.__generate_id()

        self.to_run = to_run
        self.on_success = on_success
        self.on_failure = on_failure
        self.run_date = run_date

    def trigger(self) -> bool:
        """Runs `to_run` and `on_success`/`on_failure` based on retuned value of `to_run`

        Returns
        -------
        bool
            if task was runned succesfully
        """
        is_success = False
        try:
            is_success = self.to_run()
        except Exception:
            print("[Task id: {}}] TASK FAILED: exception cougth".format(self.id))
        print("[Task] Task: " + self.id +
              " RUNNED" if is_success else " FAILED")

        if is_success and self.on_success != None:
            self.on_success()
        elif not is_success and self.on_failure != None:
            self.on_failure()

        return is_success

    def __generate_id(self, length: int = 16) -> str:
        """Generates randome string of length

        Paramaters
        ----------
        length: `int`, default `16`
            length of retuned string

        Returns
        -------
        `str`
            random generated id
        """
        return ''.join(random.SystemRandom().choice(string.ascii_lowercase + string.digits) for _ in range(length))

    def __eq__(self, other: 'Task') -> bool:
        return self.id == other.id

    def __str__(self) -> str:
        return "(Task) { " + "id: " + str(self.id) + " }"



from abc import abstractmethod
from typing import Any, Callable, Optional
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.remote.webelement import WebElement

from ..FormComponentType import FormComponentType
from .MicrosoftFormComponentType import MicrosoftFormComponentType
from ..FormComponent import FormComponent


class MicrosoftFormComponent(FormComponent):
    """Used as a super class to create more specific class components

    Paramaters
    ----------
    web_element: `'WebElement'`, default `None`
        web element associated with the form component
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        super(MicrosoftFormComponent, self).__init__(web_element=web_element)

    @abstractmethod
    def fill_in(self, response: Any):
        """Abstract method, used to be ovveride by more specific components"""
        ...

    def get_type(c: 'WebElement') -> FormComponentType:
        """Get the element type

        Paramaters
        ----------
        c: `'WebElement'`
            web element to find the type of

        Returns
        -------
        `FormComponent`
            component type for `FormComponent`
        """
        if MicrosoftFormComponent.try_find_element(c.find_element_by_xpath, './/*[contains(@class, "select-placeholder")]') != None:
            return MicrosoftFormComponentType.SELECT
        elif MicrosoftFormComponent.try_find_element(c.find_element_by_xpath, './/*[contains(@class, "office-form-date-time-picker")]') != None:
            return MicrosoftFormComponentType.DATE_PICKER
        elif MicrosoftFormComponent.try_find_element(c.find_element_by_xpath, './/*[contains(@class, "office-form-question-choice")]') != None:
            return MicrosoftFormComponentType.RADIO
        elif MicrosoftFormComponent.try_find_element(c.find_element_by_xpath, './/*[contains(@class, "office-form-textfield")]') != None:
            return MicrosoftFormComponentType.TEXT

    def try_find_element(f: Callable[[str], Any], arg: str) -> Optional[Any]:
        """Try to find an element

        Paramaters
        ----------
        f: `Callable[[str], Any]`
            function called to find element
        arg: `str`
            argument passed to f

        Returns
        -------
        `Optional[Any]`
            the element if found
        """
        elem = None
        try:
            elem = f(arg)
        except NoSuchElementException:
            pass
        return elem



from abc import abstractmethod
from typing import List

from .FormComponent import FormComponent
from ..Source import Source


class Form:
    """Abstract class to create subclass and inherit needed methods and instance variables

    Paramaters
    ----------
    url: `str`, default `""`
        url of form
    source: `'Source'`
        source to login and have other functionality (if needed)
    components: `List['FormComponent']`
        components composing form
    """

    def __init__(self, url: str = "", source: 'Source' = None, components: List['FormComponent'] = []) -> None:
        self.url = url
        self.source = source
        self.components = components

    @abstractmethod
    def find_components(self, *args, **kwargs) -> List['FormComponent']:
        """Find and parse automatically form components

        Returns
        -------
        `List['FormComponent']`
            list of parsed components
        """
        ...

    @abstractmethod
    def fill_in(self, *args, **kwargs) -> None:
        """Fill in form

        Returns
        -------
        `None`
        """
        ...

    @abstractmethod
    def send(self, *args, **kwargs) -> None:
        """Sends form

        Returns
        -------
        `None`
        """
        ...



from typing import Any, List
from selenium.webdriver.remote.webelement import WebElement

from ...FormComponentType import FormComponentType
from ..MicrosoftFormComponent import MicrosoftFormComponent


class Select(MicrosoftFormComponent):
    """Create a Microsoft form Select (drop down select) component

    Paramaters
    ----------
    web_element: `'WebElement'`, default `None`
        web element associated with form component    
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        super().__init__(web_element=web_element)

    def fill_in(self, response: str):
        """Select the right answer

        Paramaters
        ----------
        response: `str`
            the right answer
            (if options are: `["Option1", "Option 2"] you will` pass `"Option 1"` if it is the right one)
        """
        to_open_container = self.web_element.find_element_by_xpath(
            './/*[contains(@class, "select-placeholder")]')

        to_open_container.click()

        answers_container = self.web_element.find_element_by_xpath(
            './/*[contains(@class, "select-option-menu-container")]')

        answer_box = answers_container.find_element_by_xpath(
            ".//*[contains(text(), '{}')]".format(response))

        answer_box.click()



from typing import Any
from selenium.webdriver.remote.webelement import WebElement

from ..MicrosoftFormComponent import MicrosoftFormComponent


class Text(MicrosoftFormComponent):
    """Create a Microsoft form Text field component

    Paramaters
    ----------
    web_element: `'WebElement'`, default `None`
        web element associated with form component    
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        super().__init__(web_element=web_element)

    def fill_in(self, response: str):
        """Fills in the text field

        Paramaters
        ----------
        response: `str`
            the text to fill in the text field with
        """
        text_field = self.web_element.find_element_by_class_name(
            'office-form-textfield')
        input = text_field.find_element_by_xpath('.//input')
        input.clear()
        input.send_keys(response)



import os
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

default_store_path = os.path.join(os.getcwd(), 'store')
default_token_file = 'o365_token.txt'


def default_driver(path_to_store: str, path_to_driver: str):
    options = Options()
    options.add_argument(
        "user-data-dir={}/chrome_data".format(path_to_store))
    return webdriver.Chrome(
        path_to_driver, options=options)


def default_callback():
    return True



from abc import abstractmethod
from typing import Any
from selenium.webdriver.remote.webelement import WebElement

from .FormComponentType import FormComponentType


class FormComponent:
    """Abstract class to create specific form components classes.

    Paramaters
    ----------
    web_element: `'WebElement'`
        selenium web element of the form field
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        self.web_element = web_element

    @abstractmethod
    def fill_in(self, response: Any):
        """Complete the form component (for instance, fill in a text field)

        Paramaters
        ----------
        response: `Any`
            the response used to fill in the form component
        """
        ...

    @abstractmethod
    def get_type(c: 'WebElement') -> FormComponentType:
        """Gets the type of the web element (the types should be subclasses of `FormComponent`)

        Paramaters
        ----------
        c: `'WebElement'`
            web element to find the type of

        Returns
        -------
        `FormComponentType`
            the type of the form component
        """
        ...



from time import sleep
from typing import Any, List

from ...Source import Source
from ...MicrosoftSource import MicrosoftSource
from ..Form import Form
from ..FormComponent import FormComponent
from .components.Select import Select
from .components.Radio import Radio
from .components.Text import Text
from .components.DatePicker import DatePicker
from .MicrosoftFormComponent import MicrosoftFormComponent
from .MicrosoftFormComponentType import MicrosoftFormComponentType


class MicrosoftForm(Form):
    """Create a microsoft form

    Paramaters
    ----------
    url: `str`, default `""`
        form url
    source: `'Source'`, default `None`
        source used to login
    components: `List['FormComponent']`, default `[]`
        form components, if not passed `find_components` will be call to retireve them
    email_confirmation: `bool`, default `True`
        send email confirmation on when form is sent
    """

    def __init__(self, url: str = "", source: 'Source' = None, components: List['FormComponent'] = [], email_confirmation: bool = True) -> None:
        super(MicrosoftForm, self).__init__(url, components=components,
                                            source=source)
        self.url = url
        self.email_confirmation = email_confirmation
        if components == []:
            self.login_if_needed()
            self.components = self.find_components()

    def login_if_needed(self):
        """Logins using current source (Microsoft Source)

        Returns
        -------
        `None`
        """
        if isinstance(self.source, MicrosoftSource):
            self.source.driver.get(self.url)

            sleep(4)

            if self.source.needs_login():
                self.source.login()

            sleep(4)

    def fill_in(self, responses: List[Any]):
        """Uses the answers to fill in the form

        Paramaters
        ----------
        responses: `List[Any]`
            array of responses (answers needed)
        Returns 
        -------
        None
        """
        for i, c in enumerate(self.components):
            c.fill_in(response=responses[i])
        if self.email_confirmation:
            self.__select_email_check_box()

        sleep(2)

        self.send()

    def send(self):
        """Sends form"""
        if len(self.components) >= 1:
            submit_btn = self.source.driver.find_element_by_xpath(
                './/*[contains(@class, "__submit-button__")]')
            submit_btn.click()

    def __select_email_check_box(self):
        """Select checkbox to recive email confirmation"""
        try:
            container = self.source.driver.find_element_by_class_name(
                'office-form-email-receipt-checkbox')
            check_box = container.find_element_by_css_selector(
                'input[type=checkbox]')
            check_box.click()
        except:
            pass

    def find_components(self) -> List['FormComponent']:
        """Finds and parses all components in the form

        Returns
        -------
        `List['FormComponent']`
            form components found and parsed
        """
        form_container = self.source.driver.find_element_by_class_name(
            'office-form-question-body')
        childs = form_container.find_elements_by_xpath(
            './/*[contains(@class, "__question__")]')
        components = []
        for c in childs:
            type = MicrosoftFormComponent.get_type(c)

            if type == MicrosoftFormComponentType.SELECT:
                components.append(Select(web_element=c))
            elif type == MicrosoftFormComponentType.DATE_PICKER:
                components.append(DatePicker(web_element=c))
            elif type == MicrosoftFormComponentType.TEXT:
                components.append(Text(web_element=c))
            elif type == MicrosoftFormComponentType.RADIO:
                components.append(Radio(web_element=c))

        return components



from datetime import datetime
from typing import Callable, List, Optional
from apscheduler.schedulers.background import BackgroundScheduler

from .deafults import default_callback
from .Source import Source
from .Event import Event


class Calendar:
    """
    Create a new Calendar that will execute Event's Tasks

    Paramaters
    ----------
    from_source: bool, default True
        indicates if calendar should use source to fetch events
    events: Optional[List['Event']], default `[]`
        list of events (used if from_source=False)
    source: Optional['Source'], default None
        source used to fetche events from API
    filter: Optional[Callable[['Event'], bool]], default, `deafult_callback` 
        callback function used to filter events fetched using source from API
    map: Optional[Callable[['Event'], 'Event']], deafult `deafult_callback`
        callback function used to modify events fetched using source from API

    Raises
    ------
    Exception
        If from_source=True and source=None
    """

    update_events = False

    def __init__(self, from_source: bool = True, events: Optional[List['Event']] = [], source: Optional['Source'] = None, filter: Optional[Callable[['Event'], bool]] = default_callback, map: Optional[Callable[['Event'], 'Event']] = default_callback) -> None:
        self.filter = filter
        self.map = map
        self.from_source = from_source
        self.source = source
        self.scheduler = BackgroundScheduler()
        self.previous = []

        if from_source:
            if source == None:
                raise Exception('Need to provide Source')
            else:
                self.events = source.load_events(
                    filter=filter, map=map)
        else:
            if events != []:
                self.events = events

    def update(self) -> None:
        """Updates calendar by calling `source.load_events(filter=self.filter, map=self.map)` (`if source != None and from_source=True`) and
        updates scheduler with new fetched events (if any)

        Returns
        -------
        None
        """

        if self.source == None or not self.from_source:
            self.update_scheduler()
            return

        if Calendar.update_events:
            # self.previous = self.events
            self.events = self.source.load_events(
                filter=self.filter, map=self.map)

        else:
            Calendar.update_events = True
        self.update_scheduler()

    def update_scheduler(self) -> None:
        """Add new events to scheduler queue

        Returns
        -------
        None
        """

        if len(self.events) < 1 or self.events == self.previous:
            return

        tmp = self.events
        print("[Calendar] Calculating tasks...")
        for event in self.events:
            if len([e == event for e in self.previous]) == 0:
                for task in event.tasks:
                    self.scheduler.add_job(trigger='date', run_date=datetime.strftime(
                        task.run_date, '%Y-%m-%d %H:%M:%S'), func=task.trigger)
                print('[Calendar] ' + str(event) + " ADDED TO QUEUE")
        self.previous = tmp

    def listen(self):
        """Listen for tasks to be runned

        Returns
        -------
        None
        """
        self.scheduler.start()



__version__ = '0.0.1'

from .Task import Task
from .Calendar import Calendar
from .Event import Event
from .Source import Source
from .MicrosoftSource import MicrosoftSource
from .form.Form import Form
from .form.FormComponent import FormComponent
from .deafults import *



from abc import abstractmethod
from typing import List
from selenium.webdriver.chrome.webdriver import WebDriver
from .deafults import default_driver, default_store_path

from .Event import Event


class Source:
    """Abstract class used as super class and to create custom sources classes

    Paramaters
    ----------
    path_to_driver: `str`
        required path to chrome driver (default)
    driver: `'WebDriver'`, default `None`
        optional driver used instead of the deafult one
    path_to_store: `str`, default `""`
        path to file storage (folder storing tokens, driver data...)        
    """

    def __init__(self, path_to_driver: str, driver: 'WebDriver' = None, path_to_store: str = "") -> None:
        if path_to_store == "":
            self.path_to_store = default_store_path
        else:
            self.path_to_store = path_to_store

        self.path_to_driver = path_to_driver
        if driver == None:
            self.driver = default_driver(
                path_to_store=self.path_to_store, path_to_driver=self.path_to_driver)
        else:
            self.driver = driver

    @abstractmethod
    def load_events(self, *args, **kwargs) -> List['Event']:
        """Load events from external source (an API for instance)

        Returns
        `List['Event']`
            list of parsed events
        """
        ...

    @abstractmethod
    def login(self, *args, **kwargs) -> str:
        """Handle authentication to API/External Source.

        Returns
        -------
        str
            new url when authenticated
        """
        ...



from ..FormComponentType import FormComponentType


class MicrosoftFormComponentType(FormComponentType):
    """FOrmComponentType specific for Microsoft form"""
    SELECT = 'SELECT'
    DATE_PICKER = 'DATE_PICKER'
    RADIO = 'RADIO'
    TEXT = 'TEXT'



from setuptools import setup
from auto_events.__init__ import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='auto_events',
    version=__version__,
    url="https://github.com/fedecech/auto_events",
    author="Federico Cecchinato",
    author_email="federicocech@gmail.com",  # use icloud fake email
    description="Automate tasks easily using python",
    long_description=long_description,
    license='MIT',
    packages=['auto_events', 'auto_events.form',
              'auto_events.form.microsoft', 'auto_events.form.microsoft.components'],
    install_requires=['selenium==3.141.0',
                      'O365==2.0.16', 'apscheduler==3.8.0'],
    long_description_content_type="text/markdown",
    python_requires=">=3.8",
)



from selenium.webdriver.remote.webelement import WebElement

from ..MicrosoftFormComponent import MicrosoftFormComponent


class DatePicker(MicrosoftFormComponent):
    """Create a Microsoft form date picker component

    Paramaters
    ----------
    web_element: `'WebElement'`, default `None`
        web element associated with form component    
    """

    def __init__(self, web_element: 'WebElement' = None) -> None:
        super().__init__(web_element=web_element)

    def fill_in(self, response: str):
        """Select the date passed

        Paramaters
        ----------
        response: `str`
            the date to select in string format
        """
        input = self.web_element.find_element_by_xpath('.//input')
        input.click()
        input.click()
        input.clear()
        input.send_keys(response)

        body = self.web_element.find_element_by_xpath('//body')
        body.click()
